package de.towerdefence.server.server;

import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.*;

public class ServerWebsocketHandler extends TextWebSocketHandler {
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
    private final Map<WebSocketSession, ScheduledFuture<?>> sessionTaskMap = new ConcurrentHashMap<>();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        ScheduledFuture<?> scheduledTask = scheduler.scheduleAtFixedRate(
                () -> sendCurrentTime(session),
                0,
                1,
                TimeUnit.MILLISECONDS
        );
        sessionTaskMap.put(session, scheduledTask);
    }

    @Override
    public void handleTextMessage(WebSocketSession session, TextMessage message) {
        try {
            String responseMessage = "You are Connected to the Tower Defence Server Websocket";
            session.sendMessage(new TextMessage(responseMessage));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void sendCurrentTime(WebSocketSession session) {
        ScheduledFuture<?> task = sessionTaskMap.get(session);
        try {
            session.sendMessage(new TextMessage(String.valueOf(System.currentTimeMillis())));
        } catch (IllegalStateException | IOException e) {
            task.cancel(true);
            sessionTaskMap.remove(session);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
